@@ -7,7 +7,8 @@ use VIC::Grammar;
 use VIC::PIC;
 
 our $Debug = 0;
-our $VERSION = '0.02';
+
+our $VERSION = '0.03';
 $VERSION = eval $VERSION;
 
 sub compile {
@@ -17,6 +18,7 @@ sub compile {
         grammar => VIC::Grammar->new,
         receiver => VIC::PIC->new(pic_override => $pic),
         debug => $Debug,
+        throw_on_error => 1,
     );
 
     $parser->parse($input);
@@ -28,7 +30,7 @@ sub compile {
 
 =head1 NAME
 
-VIC - A Viciously Simple Syntax for PIC
+VIC - A Viciously Simple Syntax for PIC Microcontrollers
 
 =head1 SYNOPSIS
 
