@@ -2,11 +2,12 @@ package VIC::Parser;
 use strict;
 use warnings;
 
-use base 'Pegex::Parser';
-
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 $VERSION = eval $VERSION;
 
+use Pegex::Base;
+extends 'Pegex::Parser';
+
 sub match_ref {
     my ($self, $ref, $parent) = @_;
     # create a stack to track who is the parent of the current element being
